package com.sun.java.swing.plaf.windows;

import javax.swing.MenuElement;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.JRootPane;
import javax.swing.event.ChangeListener;
import sun.swing.StringUIClientPropertyKey;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Component;
import sun.swing.SwingUtilities2;
import javax.swing.PopupFactory;
import javax.swing.Popup;
import javax.swing.JPopupMenu;
import javax.swing.MenuSelectionManager;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.plaf.basic.BasicPopupMenuUI;

public class WindowsPopupMenuUI extends BasicPopupMenuUI
{
    static WindowsPopupMenuUI.MnemonicListener mnemonicListener;
    static final Object GUTTER_OFFSET_KEY;
    
    public WindowsPopupMenuUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new WindowsPopupMenuUI();
    }
    
    public void installListeners() {
        super.installListeners();
        if (!UIManager.getBoolean("Button.showMnemonics") && WindowsPopupMenuUI.mnemonicListener == null) {
            WindowsPopupMenuUI.mnemonicListener = new WindowsPopupMenuUI.MnemonicListener();
            MenuSelectionManager.defaultManager().addChangeListener(WindowsPopupMenuUI.mnemonicListener);
        }
    }
    
    @Override
    public Popup getPopup(JPopupMenu contents, int x, int y) {
        return PopupFactory.getSharedInstance().getPopup(contents.getInvoker(), contents, x, y);
    }
    
    static int getTextOffset(JComponent component) {
        int n = -1;
        Object clientProperty = component.getClientProperty(SwingUtilities2.BASICMENUITEMUI_MAX_TEXT_OFFSET);
        if ((clientProperty instanceof Integer)) {
            int intValue = (int)clientProperty;
            int x = 0;
            Component component2 = component.getComponent(0);
            if (component2 != null) {
                x = component2.getX();
            }
            n = intValue + x;
        }
        return n;
    }
    
    static int getSpanBeforeGutter() {
        return 3;
    }
    
    static int getSpanAfterGutter() {
        return 3;
    }
    
    static int getGutterWidth() {
        int width = 2;
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            width = xp.getSkin(null, TMSchema.Part.MP_POPUPGUTTER).getWidth();
        }
        return width;
    }
    
    private static boolean isLeftToRight(JComponent component) {
        boolean leftToRight = true;
        for (int n = component.getComponentCount() - 1; n >= 0 && leftToRight; leftToRight = component.getComponent(n).getComponentOrientation().isLeftToRight(), --n) {}
        return leftToRight;
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        XPStyle xp = XPStyle.getXP();
        if (WindowsMenuItemUI.isVistaPainting(xp)) {
            xp.getSkin(c, TMSchema.Part.MP_POPUPBACKGROUND).paintSkin(g, 0, 0, c.getWidth(), c.getHeight(), TMSchema.State.NORMAL);
            int textOffset = getTextOffset(c);
            if (textOffset >= 0 && isLeftToRight(c)) {
                XPStyle.Skin skin = xp.getSkin(c, TMSchema.Part.MP_POPUPGUTTER);
                int gutterWidth = getGutterWidth();
                int i = textOffset - getSpanAfterGutter() - gutterWidth;
                c.putClientProperty(WindowsPopupMenuUI.GUTTER_OFFSET_KEY, Integer.valueOf(i));
                Insets insets = c.getInsets();
                skin.paintSkin(g, i, insets.top, gutterWidth, c.getHeight() - insets.bottom - insets.top, TMSchema.State.NORMAL);
            }
            else if (c.getClientProperty(WindowsPopupMenuUI.GUTTER_OFFSET_KEY) != null) {
                c.putClientProperty(WindowsPopupMenuUI.GUTTER_OFFSET_KEY, null);
            }
        }
        else {
            super.paint(g, c);
        }
    }
    
    static {
        WindowsPopupMenuUI.mnemonicListener = null;
        GUTTER_OFFSET_KEY = new StringUIClientPropertyKey("GUTTER_OFFSET_KEY");
    }
    
    static class MnemonicListener implements ChangeListener
    {
        JRootPane repaintRoot;
        
        MnemonicListener() {
            this.repaintRoot = null;
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            MenuElement[] selectedPath = ((MenuSelectionManager)changeEvent.getSource()).getSelectedPath();
            if (selectedPath.length == 0) {
                if (!WindowsLookAndFeel.isMnemonicHidden()) {
                    WindowsLookAndFeel.setMnemonicHidden(true);
                    if (this.repaintRoot != null) {
                        WindowsGraphicsUtils.repaintMnemonicsInWindow(SwingUtilities.getWindowAncestor(this.repaintRoot));
                    }
                }
            }
            else {
                Component invoker = (Component)selectedPath[0];
                if ((invoker instanceof JPopupMenu)) {
                    invoker = ((JPopupMenu)invoker).getInvoker();
                }
                this.repaintRoot = SwingUtilities.getRootPane(invoker);
            }
        }
    }
}
